﻿using System;
using System.Data.SQLite;
using System.Drawing;
using System.IO.Ports;
using System.Text;
using System.Windows.Forms;

namespace DirtyWordsFeedback
{
    class App
    {
        // 数据缓存路径
        public static string tempPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
        // 串口类
        SerialPort serialPort = null;
        // 打开串口标志位
        bool portOpen = false;
        // 数据库连接
        private static SQLiteConnection connection = null;

        private static App app;

        private App() { }

        public static App getApp()
        {
            if(app == null)
            {
                app = new App();

                bool dataExists = System.IO.File.Exists(tempPath + "/Data.sqlite");
                if (!dataExists)
                {
                    SQLiteConnection.CreateFile(tempPath + "/Data.sqlite");
                    Console.WriteLine("Data.sqlite");
                }
                connection = new SQLiteConnection("Data Source=" + tempPath + "/Data.sqlite;Version=3;Pooling=true;FailIfMissing=false;");
                connection.Open();
                if (!dataExists)
                {
                    new SQLiteCommand("create table data (id INTEGER PRIMARY KEY AUTOINCREMENT, content varchar(255), level  varchar(16))", connection).ExecuteNonQuery();
                    Console.WriteLine("create table");
                }
            }
            return app;
        }

        public SerialPort SerialPort
        {
            set
            {
                if(value == null)
                {
                    portOpen = false;
                    MainForm.createMainForm().portStatus.Text = "未连接";
                    MainForm.createMainForm().portStatus.LinkColor = Color.Red;
                    MainForm.createMainForm().portStatus.LinkBehavior = LinkBehavior.SystemDefault;
                }
                else
                {
                    portOpen = true;
                    MainForm.createMainForm().portStatus.Text = "已连接";
                    MainForm.createMainForm().portStatus.LinkColor = Color.Green;
                    MainForm.createMainForm().portStatus.LinkBehavior = LinkBehavior.NeverUnderline;
                }
                serialPort = value;
            }
            get
            {
                return serialPort;
            }
        }

        public bool PortOpen
        {
            set
            {
                portOpen = value;
            }
            get
            {
                return portOpen;
            }
        }

        public SQLiteConnection Connection
        {
            get
            {
                return connection;
            }
        }

        public void dataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                // Comm.BytesToRead 中为要读入的字节长度
                int len = serialPort.BytesToRead;
                Byte[] readBuffer = new Byte[len];
                // 将数据读入缓存
                serialPort.Read(readBuffer, 0, len);
                // 处理 readBuffer 中的数据，自定义处理过程
                string msg = Encoding.Default.GetString(readBuffer, 0, len);
                Console.WriteLine(msg);
            }
            catch (Exception ex)
            {
                MessageBox.Show("接收返回消息异常！具体原因：" + ex.Message, "错误提示");
            }
        }
    }
}
