﻿using System;
using System.Collections.Generic;
using System.Data.SQLite;
using System.Drawing;
using System.IO.Ports;
using System.Windows.Forms;

namespace DirtyWordsFeedback
{
    public partial class MainForm : Form
    {
        App app = App.getApp();
        SerialTool serialTool = SerialTool.getSerialTool();
        private bool noDevice = true;
        private SQLiteHelper sql = new SQLiteHelper("Data Source=" + App.tempPath + "/Data.sqlite;Version=3;Pooling=true;FailIfMissing=false;");

        private static MainForm mainForm;
        private static readonly object SynObject = new object();
        public static MainForm createMainForm()
        {
            lock (SynObject)
            {
                if (mainForm == null || mainForm.IsDisposed)
                {
                    mainForm = new MainForm();
                }
                return mainForm;
            }
        }

        private MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            //inputTip.Location = new Point((form.Width - inputTip.Width) / 2, inputTip.Location.Y);
            //inputText.Location = new Point((form.Width - inputText.Width) / 2, inputText.Location.Y);
            //submit.Location = new Point((form.Width - submit.Width) / 2, submit.Location.Y);

            portStatus.Text = "未连接";
            portStatus.LinkColor = Color.Red;
            portStatus.LinkBehavior = LinkBehavior.SystemDefault;
            loadDevice();

            mediaPlayer.uiMode = "none";
            mediaPlayer.settings.autoStart = true;          // 取消自动播放
            mediaPlayer.settings.setMode("loop", false);    // 取消循环播放
            mediaPlayer.enableContextMenu = false;          // 禁用右键菜单
            mediaPlayer.settings.volume = 100;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!noDevice)
            {
                MessageBoxButtons messageBoxButtons = MessageBoxButtons.OKCancel;
                DialogResult dr = MessageBox.Show("确定退出程序吗？", "提示", messageBoxButtons);
                if (dr == DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                submit_Click(submit, null);
                inputText.Focus();
            }
        }

        private void data_Click(object sender, EventArgs e)
        {
            new DataForm().ShowDialog();
        }

        private void settings_Click(object sender, EventArgs e)
        {
            SettingsForm.createSettingsForm().ShowDialog();
        }

        private void about_Click(object sender, EventArgs e)
        {
            new AboutBox().ShowDialog();
        }

        private void portStatus_Click(object sender, EventArgs e)
        {
            if (!app.PortOpen)
            {
                SettingsForm.createSettingsForm().ShowDialog();
            }
        }

        private void submit_Click(object sender, EventArgs e)
        {
            string sql = "", resultText = "", sendText = "";
            if (Properties.Settings.Default.matchAccurate)
            {
                sql = "SELECT * FROM data WHERE content='" + inputText.Text.Trim() + "' order by id desc";
            }
            else
            {
                sql = "SELECT * FROM data WHERE content like '%" + inputText.Text.Trim() + "%' order by id desc";
            }
            SQLiteCommand command = new SQLiteCommand(sql, app.Connection);
            SQLiteDataReader reader = command.ExecuteReader();
            while (reader.Read())
            {
                resultText = "内容: " + reader["content"] + "  分类: " + reader["level"];
                switch (reader["level"].ToString())
                {
                    case "轻度":
                        sendText = "1";
                        mediaPlayer.URL = Properties.Settings.Default.videoLevel1;
                        break;
                    case "中度":
                        sendText = "2";
                        mediaPlayer.URL = Properties.Settings.Default.videoLevel2;
                        break;
                    case "重度":
                        sendText = "3";
                        mediaPlayer.URL = Properties.Settings.Default.videoLevel3;
                        break;
                    default:
                        sendText = "0";
                        break;
                }
                Console.WriteLine(resultText);
            }
            inputText.Text = "";
            if(resultText == "")
            {
                result.Text = "数据库中未定义";
                sendText = "0";
            }
            else
            {
                result.Text = resultText;
            }
            
            // 写串口数据
            if (app.PortOpen)
            {
                try
                {
                    app.SerialPort.WriteLine(sendText);
                }
                catch (Exception)
                {
                    MessageBox.Show("发送数据时发生错误！", "错误提示"); return;
                }
            }
            else
            {
                MessageBox.Show("硬件未连接或配置错误！", "错误提示"); return;
            }
        }

        private void loadDevice()
        {
            List<string> portList = serialTool.findPort();
            if (portList.Count == 0)
            {
                MessageBoxButtons messageBoxButtons = MessageBoxButtons.OK;
                DialogResult dr = MessageBox.Show("没有找到可用硬件！\n请确保连接硬件后再打开本软件。", "错误提示", messageBoxButtons);
                if (dr == DialogResult.OK)
                {
                    this.Close();
                }
            }
            else if (portList.Count == 1)
            {
                noDevice = false;
                SerialPort serialPort = serialTool.openPort(portList[0], "9600", "8", "1", "无", app.dataReceived);
                if (serialPort != null)
                {
                    app.SerialPort = serialPort;
                    portStatus.Text = "已连接";
                    portStatus.LinkColor = Color.Green;
                    portStatus.LinkBehavior = LinkBehavior.NeverUnderline;
                }
            }
            else
            {
                noDevice = false;
                MessageBox.Show("请到设置页面配置正确串口！", "提示");
                if (!app.PortOpen)
                {
                    SettingsForm.createSettingsForm().ShowDialog();
                }
            }
        }

        // 采用 Windows 的消息机制来捕获插入的 USB 状态
        public const int WM_DEVICE_CHANGE = 0x219;
        public const int DBT_DEVICEARRIVAL = 0x8000;
        public const int DBT_DEVICE_REMOVE_COMPLETE = 0x8004;
        protected override void WndProc(ref Message m)
        {
            // 捕获USB设备的拔出消息 WM_DEVICECHANGE
            if (m.Msg == WM_DEVICE_CHANGE)
            {
                switch (m.WParam.ToInt32())
                {
                    case DBT_DEVICE_REMOVE_COMPLETE:    // USB 拔出
                        Console.WriteLine("USB 拔出");
                        break;
                    case DBT_DEVICEARRIVAL:             // USB 插入获取对应串口名称
                        Console.WriteLine("USB 插入");
                        loadDevice();
                        break;
                }
            }
            base.WndProc(ref m);
        }
    }
}
