﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Windows.Forms;

namespace DirtyWordsFeedback
{
    public partial class SettingsForm : Form
    {
        App app = App.getApp();
        SerialTool serialTool = SerialTool.getSerialTool();

        private static SettingsForm settingsForm;

        private SettingsForm()
        {
            InitializeComponent();
        }

        public static SettingsForm createSettingsForm()
        {
            if (settingsForm == null || settingsForm.IsDisposed)
            {
                settingsForm = new SettingsForm();
            }
            return settingsForm;
        }

        private void SettingsForm_Load(object sender, EventArgs e)
        {
            // 最大支持到串口10，可根据自己需求增加
            for (int i = 0; i < 10; i++)
            {
                cbxCOMPort.Items.Add("COM" + (i + 1).ToString());
            }
            cbxCOMPort.SelectedIndex = 0;

            if (app.PortOpen)
            {
                cbxCOMPort.Items.Clear();
                cbxCOMPort.Items.Add(app.SerialPort.PortName);
                cbxCOMPort.SelectedIndex = 0;
                COMOpen.Text = "断开连接";
            }
            else
            {
                COMCheck.PerformClick();
            }

            videoLevel1.Text = Properties.Settings.Default.videoLevel1;
            videoLevel2.Text = Properties.Settings.Default.videoLevel2;
            videoLevel3.Text = Properties.Settings.Default.videoLevel3;
            matchAccurate.Checked = Properties.Settings.Default.matchAccurate;
            matchFuzzy.Checked = Properties.Settings.Default.matchFuzzy;
        }

        private void COMCheck_Click(object sender, EventArgs e)
        {
            cbxCOMPort.Enabled = false;
            COMCheck.Enabled = false;
            COMOpen.Enabled = false;
            // 清除当前串口号中的所有串口名称
            cbxCOMPort.Items.Clear();
            List<string> portList = serialTool.findPort();
            if(portList.Count == 0)
            {
                MessageBox.Show("没有找到可用串口！", "错误提示");
            }
            else
            {
                portList.ForEach(
                    delegate(string portName) {
                        cbxCOMPort.Items.Add(portName);
                    });
                //使 ListBox 显示第1个添加的索引
                cbxCOMPort.SelectedIndex = 0;
            }
            cbxCOMPort.Enabled = true;
            COMCheck.Enabled = true;
            COMOpen.Enabled = true;
        }

        private void COMOpen_Click(object sender, EventArgs e)
        {
            if (app.PortOpen == false)
            {
                // 检测串口设置
                string portName = cbxCOMPort.Text.Trim();
                if (portName == "")
                {
                    MessageBox.Show("串口未设置！", "错误提示");
                    return;
                }
                // 打开串口
                SerialPort serialPort = serialTool.openPort(portName, "9600", "8", "1", "无", app.dataReceived);
                if(serialPort == null)
                {
                    // 打开串口失败
                    MessageBox.Show("串口无效或已被占用！", "错误提示");
                }
                else
                {
                    app.SerialPort = serialPort;
                    COMOpen.Text = "断开连接";
                    cbxCOMPort.Enabled = false;
                    COMCheck.Enabled = false;
                }
            }
            else
            {
                // 关闭串口
                if (serialTool.closePort(app.SerialPort))
                {
                    app.SerialPort = null;
                    COMOpen.Text = "连接设备";
                    cbxCOMPort.Enabled = true;
                    COMCheck.Enabled = true;
                }
                else
                {
                    MessageBox.Show("断开连接失败！", "错误提示");
                }
            }
        }

        private void save_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.matchAccurate = matchAccurate.Checked;
            Properties.Settings.Default.matchFuzzy = matchFuzzy.Checked;
            Properties.Settings.Default.Save();
            Properties.Settings.Default.Reload();
            this.Close();
        }

        private void chooseVideo1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "媒体文件（所有类型）|*.mp3;*.mpeg;*.wma;*.wmv;*.wav;*.avi;*.mp4|所有文件|*.*";
            ofd.ValidateNames = true;
            ofd.CheckPathExists = true;
            ofd.CheckFileExists = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                string strFileName = ofd.FileName;
                videoLevel1.Text = strFileName;
                Properties.Settings.Default.videoLevel1 = strFileName;
            }
        }

        private void chooseVideo2_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "媒体文件（所有类型）|*.mp3;*.mpeg;*.wma;*.wmv;*.wav;*.avi;*.mp4|所有文件|*.*";
            ofd.ValidateNames = true;
            ofd.CheckPathExists = true;
            ofd.CheckFileExists = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                string strFileName = ofd.FileName;
                videoLevel2.Text = strFileName;
                Properties.Settings.Default.videoLevel2 = strFileName;
            }
        }

        private void chooseVideo3_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "媒体文件（所有类型）|*.mp3;*.mpeg;*.wma;*.wmv;*.wav;*.avi;*.mp4|所有文件|*.*";
            ofd.ValidateNames = true;
            ofd.CheckPathExists = true;
            ofd.CheckFileExists = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                string strFileName = ofd.FileName;
                videoLevel3.Text = strFileName;
                Properties.Settings.Default.videoLevel3 = strFileName;
            }
        }
    }
}
